@@ -4,17 +4,26 @@ __all__ = ('common_name_mapper',)
 
 
 def common_name_mapper(fname):
-	"""this is a common name mapping function: take for example a file of name 5um3_50e-9_1e-9_0x5V_500mv_10000ns_1
-	spl is a split on '_'
-	
-	'identifier':spl[0],
-	'pulsewidth_ns':float(spl[1])*1e9,
-	'delay_ns':float(spl[2])*1e9,
-	'high_voltage_v':float(spl[3].replace('V','').replace('x','.')),
-	'preset_voltage_v':float(spl[4].replace('mv',''))/1000,
-	'preset_pulsewidth_ns':float(spl[5].replace('ns','')),
-	'filename':fname,
-	'trial':int(spl[6])
+	"""Common name mapper used in our experiment. 
+
+	args:
+		fname (str): Filename
+	 
+	Example:
+	```
+	>>>fname = '5um3_50e-9_1e-9_0x5V_500mv_10000ns_1'
+	>>> common_name_mapper(fname)
+	> {
+		'identifier':5um3,
+		'pulsewidth_ns':50,
+		'delay_ns':1,
+		'high_voltage_v':0.5,
+		'preset_voltage_v':0.5,
+		'preset_pulsewidth_ns':10000,
+		'filename':'5um3_50e-9_1e-9_0x5V_500mv_10000ns_1',
+		'trial':1
+	}
+	```
 	"""
 	string = fname.replace('.csv','')
 	spl = string.split('_')
@@ -32,6 +41,13 @@ def common_name_mapper(fname):
 		}
 	)
 
+	area_diameter_dict.update({
+			'8um': (np.pi*(8/2)**2, 8),
+			'4um': (np.pi*(4/2)**2, 4),
+			'6um': (np.pi*(6/2)**2, 6),
+		}
+	)
+
 	area, diameter = area_diameter_dict[spl[0].split('um')[0] + 'um']
 
 	out = dict({
